unit unOzTimer;

interface
uses windows,messages,classes,sysutils;

type

  TOnLogException = procedure (s:string) of object;
  TozTimerEvent = procedure (Sender: TObject) of object;

  TozTimer = class(TObject)
  private
    FInterval: Cardinal;
    FWindowHandle: HWND;
    FOnTimer: TozTimerEvent;
    FEnabled: Boolean;
    procedure UpdateTimer;
    procedure SetEnabled(Value: Boolean);
    procedure SetInterval(Value: Cardinal);
    procedure SetOnTimer(Value: TozTimerEvent);
    procedure WndProc(var Msg: TMessage);
  protected
    procedure Timer; dynamic;
  public
    tag : integer;
    onLogException : TOnLogException;
    constructor Create;
    destructor Destroy; override;
  published
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Interval: Cardinal read FInterval write SetInterval default 1000;
    property OnTimer: TozTimerEvent read FOnTimer write SetOnTimer;
  end;

implementation

constructor TozTimer.Create;
begin
  tag := 0;
  FEnabled := True;
  FInterval := 1000;
  FWindowHandle := AllocateHWnd(WndProc);
end;

destructor TozTimer.Destroy;
begin
  FEnabled := False;
  UpdateTimer;
  DeallocateHWnd(FWindowHandle);
end;

procedure TozTimer.WndProc(var Msg: TMessage);
begin
  with Msg do
    if Msg = WM_TIMER then
      try
        Timer;
      except
        on e:exception do begin
           if assigned(onLogException) then onLogException(e.message);
        end;
      end
    else
      Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
end;

procedure TozTimer.UpdateTimer;
begin
  KillTimer(FWindowHandle, 1);
  if (FInterval <> 0) and FEnabled and Assigned(FOnTimer) then
    if SetTimer(FWindowHandle, 1, FInterval, nil) = 0 then begin
      if assigned(onLogException) then onLogException('Cannot set timer, out of resoureces');
    end;
end;

procedure TozTimer.SetEnabled(Value: Boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    UpdateTimer;
  end;
end;

procedure TozTimer.SetInterval(Value: Cardinal);
begin
  if Value <> FInterval then
  begin
    FInterval := Value;
    UpdateTimer;
  end;
end;

procedure TozTimer.SetOnTimer(Value: TozTimerEvent);
begin
  FOnTimer := Value;
  UpdateTimer;
end;

procedure TozTimer.Timer;
begin
  if Assigned(FOnTimer) then FOnTimer(Self);
end;

end.
